#!/bin/bash

if [ "$1" == "-leaveFiles" ]; then
    leave="y"
    shift
else
    leave="n"
fi

if [ "$1" == "-align" ]; then
    align="y"
    shift
else
    align="n"
fi

if [ "$1" == "-bino" ]; then
    type="bino"
    align="y"
    shift
else
    type="micro"
fi

if [ "$1" == "-mag" ]; then
    mag="$2"
    shift 2
else
    mag=""
fi

if [ "$1" == "-gaussian" ]; then
    contrast_edge_scale=""
    shift
else
    contrast_edge_scale='--contrast-edge-scale=0.3:1000%:3.0'
fi

if [ "$1" == "-contrast-window-size" ]; then
    contrast_window_size="--contrast-window-size=$2"
    shift 2
else
    contrast_window_size=""
fi

if [ $# -lt 2 ]; then
    echo "USAGE: $0 < -leaveFiles -align -bino -mag -gaussian -contrast-window-size size> outFile ( -all | inFiles)" >&2
    exit 1
fi

. $HOME/bin/locale.sh

outFile=$1
shift 1

if [ -r "$outFile" ]; then
    echo "outFile \"$outFile\" exists, continue anyway(y/n)?" >&2
    read ans
    if [ "$ans" != "y" ]; then
	exit 1
    fi
fi

if [ "$type" == "bino" ]; then
    if [ "$1" == "-all" ]; then
	echo "ERROR: -all doesn't work in combination with -type bino" >&2
	exit 1
    fi
    inFiles="$@"
    align_image_stack -a X $inFiles
    files="X*.tif"
    sfiles=$(for file in $files; do echo $file; done |sort)

    echo enfuse "$contrast_edge_scale" "$contrast_window_size" --exposure-weight=0 --saturation-weight=0 --contrast-weight=1 --hard-mask --output=$outFile $sfiles
    if ! enfuse "$contrast_edge_scale" "$contrast_window_size" --exposure-weight=0 --saturation-weight=0 --contrast-weight=1 --hard-mask --output=$outFile $sfiles; then
	exit 1
    fi

    if [ -n "$mag" ]; then
	mag=$(calcExifScale $type $mag)
	mag="-ImageDescription=$mag"
    fi
    exiftool -Make=Canon $mag -overwrite_original $outFile

    if [ "$leave" == "n" ]; then
	rm $inFiles $files
    fi

elif [ "$type" == "micro" ]; then
    if [ "$1" == "-all" ]; then
	if [ $(echo Image_*.jpg|wc -w) -eq 1 ]; then
	    if [ "Image_*.jpg" == ''Image_*.jpg'' ]; then
		if ! mv $ucviewTmpDir/Image_*.jpg .; then
		    echo "ERROR: no files, neither $(pwd)/Image_*.jpg nor $ucviewTmpDir/Image_*.jpg" >&2
		    exit 1
		fi
	    fi
	fi
	inFiles=$(for file in Image_*.jpg; do echo $file; done |sort)
    else
	inFiles="$@"
    fi

    if [ "$align" == "y" ]; then
	align_image_stack -a X $inFiles
	files=$(for file in "X*.tif"; do echo $file; done |sort)

	echo enfuse "$contrast_edge_scale" "$contrast_window_size" --exposure-weight=0 --saturation-weight=0 --contrast-weight=1 --hard-mask --output=$outFile $files
	if ! enfuse "$contrast_edge_scale" "$contrast_window_size" --exposure-weight=0 --saturation-weight=0 --contrast-weight=1 --hard-mask --output=$outFile $files; then
	    exit 1
	fi

	if [ "$leave" == "n" ]; then
	    rm $inFiles $files
	fi
    else
	echo enfuse "$contrast_edge_scale" "$contrast_window_size" --exposure-weight=0 --saturation-weight=0 --contrast-weight=1 --hard-mask --output=$outFile $inFiles
	if ! enfuse "$contrast_edge_scale" "$contrast_window_size" --exposure-weight=0 --saturation-weight=0 --contrast-weight=1 --hard-mask --output=$outFile $inFiles; then
	    exit 1
	fi

	if [ "$leave" == "n" ]; then
	    rm $inFiles
	fi
    fi

    if [ -n "$mag" ]; then
	mag=$(calcExifScale $type $mag)
	mag="-ImageDescription=$mag"
    fi

    exiftool -Make=tisUcview $mag -overwrite_original $outFile

fi

